import Foundation
import metaoneSDK
import SwiftUI


@objc(MetaoneWalletReactNativeSdk)
class MetaoneWalletReactNativeSdk: NSObject {
    @State var subscription: metaoneSDK.Subscription?
    private var metaOneSDKManager = MetaOneSDKManager()
    
    @objc(initialize:resolve:reject:)
    func initialize(sdkConfig:NSDictionary,  resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        if let config = sdkConfig as? [String: Any] {
            metaOneSDKManager.initialize(
                sdkConfig: SDKConfig(
                    SDK_VERSION: (config["version"] as? String)!,
                    SDK_REALM: (config["sdkRealm"] as? String)!,
                    SDK_ENVIRONMENT: (config["sdkEnvironment"] as? String)!,
                    SDK_KEY: (config["sdkKey"] as? String)!,
                    SDK_CONFIG_URL: (config["sdkConfigUrl"] as? String)!,
                    SDK_PUB_PHRASE: (config["sdkApiKeyPhrase"] as? String)!,
                    SDK_API_CLIENT_REFERENCE: (config["sdkApiClientReference"] as? String)!
                )
            )
            resolve(true)
        } else {
            reject("Invalid Config", "SDK Config is not valid", nil)
        }
    }
    
    @objc func getSessionActivityStatus(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let result = metaOneSDKManager.getSessionActivityStatus()
        resolve(result.rawValue)
    }
    
    @objc(loginWithSSO:resolve:reject:)
    func loginWithSSO(accessToken:String,  resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let loginRequest = AuthApiModel.SSOLoginRequest(token: accessToken)
        let callback = M1EnqueueCallback<Bool>(
            onSuccess: { _ in
                self.metaOneSDKManager.setup(completion: {
                    resolve(true)
                })
            },
            onError: { resp in
                if !APIError.isLinkRequired(code: resp.code, responseCode: resp.responseCode) {
                    resolve(false)
                }
            }
        )
        metaOneSDKManager.login(requestData: loginRequest, callback: callback)
    }
    
    @objc func getExpireAt(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let expiredAt = metaOneSDKManager.getExpireAt()
        resolve(expiredAt)
    }
    
    @objc func refreshSession(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        metaOneSDKManager.refreshSession(callback: PromiseCallback<Bool>(resolve: resolve, reject: reject).enqueueCallback)
    }
    
    
    @objc func openWallet(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        if let result = metaOneSDKManager.openWallet(){
            resolve(result)
        } else{
            navigate(view: metaOneSDKManager.sdkView!)
            resolve(true)
        }
    }
    
    
    @objc func getColorsScheme(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let colorsJsonString = metaOneSDKManager.uiManager.getCurrentColorScheme().toJsonString()
        resolve(Helper().toDictionary(text: colorsJsonString!))
    }
    
    @objc func setColorsScheme(_ colors: NSDictionary) {
        let jsonString = Helper().toString(dictionary: colors)
        let colorScheme = AppConfig.ColorsScheme.fromJsonString(jsonString: jsonString!)
        metaOneSDKManager.uiManager.setCurrentColorScheme(colorScheme: colorScheme)
    }
    
    
    @objc func getCurrentLanguage(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let currentLanguage = metaOneSDKManager.uiManager.getCurrentLanguage()
        
        let locale: String
        switch currentLanguage {
        case  SupportedLanguage.ENGLISH_US : locale = "en-US"
        case  SupportedLanguage.VIETNAMESE : locale = "vi-VN"
        case  SupportedLanguage.INDONESIAN : locale = "in-ID"
        case  SupportedLanguage.FILIPINO : locale = "tl-PH"
        case  SupportedLanguage.CHINESE_SIMPLIFIED : locale = "zh-CN"
        case  SupportedLanguage.CHINESE_TRADITIONAL : locale = "zh-TW"
        default: locale = "en-US"
        }
        
        resolve(locale)
    }
    
    @objc func setCurrentLanguage(_ locale: String) {
        let currentLanguage: SupportedLanguage
        switch locale {
        case "en-US": currentLanguage = SupportedLanguage.ENGLISH_US
        case "vi-VN": currentLanguage = SupportedLanguage.VIETNAMESE
        case "in-ID": currentLanguage = SupportedLanguage.INDONESIAN
        case "tl-PH": currentLanguage = SupportedLanguage.FILIPINO
        case "zh-CN": currentLanguage = SupportedLanguage.CHINESE_SIMPLIFIED
        case "zh-TW": currentLanguage = SupportedLanguage.CHINESE_TRADITIONAL
        default: currentLanguage = SupportedLanguage.ENGLISH_US
        }
        metaOneSDKManager.uiManager.setCurrentLanguage(locale: currentLanguage)
    }
    
    @objc func getWallets(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    
    {
        metaOneSDKManager.apiManager.getWallets(callback: PromiseCallback<UserWalletsResponse>(resolve: resolve, reject: reject).enqueueCallback)
    }
    
    @objc func getCurrencies(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    
    {
        metaOneSDKManager.apiManager.getCurrencies(callback: PromiseCallback<UserCurrenciesResponse>(resolve: resolve, reject: reject).enqueueCallback)
    }
    
    @objc(getTransactions:assetRef:bip44:tokenAddress:limit:offset:resolve:reject:)
    func getTransactions(
        walletId:String?,
        assetRef:String?,
        bip44:String?,
        tokenAddress:String?,
        limit:String,
        offset:String,
        resolve:@escaping RCTPromiseResolveBlock,
        reject: @escaping RCTPromiseRejectBlock
    ){
        metaOneSDKManager.apiManager.getTransactions(
            walletId: walletId,
            assetRef: assetRef,
            bip44: bip44,
            tokenAddress: tokenAddress,
            page: Int(limit),
            offset: Int(offset),
            callback: PromiseCallback<TransactionAPIModel.TransactionsResponse>(resolve: resolve, reject: reject).enqueueCallback
        )
        
    }
    @objc(getNFTs:searchString:limit:offset:resolve:reject:)
    func getNFTs(
        walletId: String?, searchString: String?, limit: String, offset: String, resolve:@escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock
    ){
        metaOneSDKManager.apiManager.getNfts(
            walletId: walletId,
            queryString: searchString,
            limit: Int(limit)!,
            offset: Int(offset)!,
            callback: PromiseCallback<UserNFTsResponse>(resolve: resolve, reject: reject).enqueueCallback
        )
    }
    
    @objc func getUserContacts(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    @objc(getUserContactWithId:resolve:reject:)
    func getUserContactWithId(contactId:String, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    @objc(addUserContact:resolve:reject:)
    func addUserContact(contact:NSDictionary, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    @objc(updateUserContact:contact:resolve:reject:)
    func updateUserContact(contactId: String, contact:NSDictionary, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    
    @objc(deleteUserContact:resolve:reject:)
    func deleteUserContact(contactId: String, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    
    @objc func logout()
    {
        metaOneSDKManager.logout()
    }
    
    
    func navigate (view:AnyView) {
        DispatchQueue.main.async {
            let hostingController = UIHostingController(rootView: view)
            
            if let subscription = self.subscription  {
                subscription.unsubscribe()
            }
            
            self.subscription = self.metaOneSDKManager.events.subscribe{ event in
                if  event == .hideSDKUI {
                    UIView.animate(withDuration: 0.3, animations: {
                        hostingController.view.frame = CGRect(x: UIScreen.main.bounds.width, y: 0, width: hostingController.view.frame.width, height: hostingController.view.frame.height)
                    }, completion: {finished in
                        hostingController.willMove(toParent: nil)
                        hostingController.view.removeFromSuperview()
                        hostingController.removeFromParent()
                    })
                }
            }
            if let rootViewController = UIApplication.shared.delegate?.window??.rootViewController {
                rootViewController.addChild(hostingController)
                rootViewController.view.addSubview(hostingController.view)
                hostingController.view.translatesAutoresizingMaskIntoConstraints = false
                NSLayoutConstraint.activate([
                    hostingController.view.topAnchor.constraint(equalTo: rootViewController.view.topAnchor),
                    hostingController.view.bottomAnchor.constraint(equalTo: rootViewController.view.bottomAnchor),
                    hostingController.view.leadingAnchor.constraint(equalTo: rootViewController.view.leadingAnchor),
                    hostingController.view.trailingAnchor.constraint(equalTo: rootViewController.view.trailingAnchor)
                ])
                hostingController.view.frame = CGRect(x: UIScreen.main.bounds.width, y: 0, width: hostingController.view.frame.width, height: hostingController.view.frame.height)
                UIView.animate(withDuration: 0.3, animations: {
                    hostingController.view.frame = CGRect(x: 0, y: 0, width: hostingController.view.frame.width, height: hostingController.view.frame.height)
                })
                hostingController.didMove(toParent: rootViewController)
                
            }
        }
    }
}
